int g_pluginId;
array<Twitch::Message@> g_messages;
bool g_showTable;
bool g_showPopup;
Twitch::Message@ g_currentMessage;

string g_sendingMessage;
string[] g_channels;
string g_currentChannel;

void Main() {
	while (!Twitch::ChannelsAdded()) yield();
	array<Twitch::ChannelState@> channels = Twitch::GetAllChannels();
	for (uint i = 0; i < channels.Length; i++) {
		g_channels.InsertLast(channels[i].m_name);
	}
	startnew(ChannelsLazyUpdate);
	if (g_channels.Length > 0) g_currentChannel = g_channels[0];
	while (true) {
		if (g_pluginId == 0) g_pluginId = Twitch::Register(0xFFF);
		array<Twitch::Message@> newMessages = Twitch::Fetch(g_pluginId);
		for (uint i = 0; i < newMessages.Length; i++) {
			g_messages.InsertLast(newMessages[i]);
		}
		yield();
	}
}

void ChannelsLazyUpdate() {
	while (!Twitch::ChannelsJoined()) yield();
	array<Twitch::ChannelState@> channels = Twitch::GetAllChannels();
	g_channels.RemoveRange(0,g_channels.Length);
	for (uint i = 0; i < channels.Length; i++) {
		g_channels.InsertLast(channels[i].m_name);
	}
}

void OnDisabled() {
	if (g_pluginId > 0) Twitch::Unregister(g_pluginId);
	g_pluginId = 0;
}

void RenderMenu() {
	if(UI::MenuItem(Icons::Wrench + " Twitch Chat Inspector", "", g_showTable)) {
		g_showTable = !g_showTable;
	}
}

string MessageTypeString(Twitch::MessageType enumValue) {
    switch (enumValue) {
        case Twitch::MessageType::ChatMessage:
            return "1 (ChatMessage)";
        case Twitch::MessageType::UserNotice:
            return "2 (UserNotice)";
        case Twitch::MessageType::UserState:
            return "4 (UserState)";
        case Twitch::MessageType::Whisper:
            return "8 (Whisper)";
        case Twitch::MessageType::ClearChat:
            return "16 (ClearChat)";
        case Twitch::MessageType::ClearMessage:
            return "32 (ClearMessage)";
        case Twitch::MessageType::HostTarget:
            return "64 (HostTarget)";
        case Twitch::MessageType::Notice:
            return "128 (Notice)";
        case Twitch::MessageType::Join:
            return "256 (Join)";
        case Twitch::MessageType::RoomState:
            return "512 (RoomState)";
        case Twitch::MessageType::GlobalUserState:
            return "1024 (GlobalUserState)";
    }
    return "Unknown";
}

void RenderInterface() {
	if (g_showTable) {
		if (UI::Begin(Icons::List + " Twitch Chat Inspector###Twitch Chat Inspector", g_showTable)) {
			bool changed = false;
			g_sendingMessage = UI::InputText(" Send message", g_sendingMessage, changed, UI::InputTextFlags::EnterReturnsTrue);
			if (changed) {
				Twitch::MessageCode sentCode = Twitch::SendMessage(g_currentChannel, g_sendingMessage);
				UI::SetKeyboardFocusHere(-1);
				g_sendingMessage = "";
			}
			if (UI::BeginCombo("Channel", g_currentChannel)) {
				for (uint i = 0; i < g_channels.Length; i++) {
					string channel = g_channels[i];
					if (UI::Selectable(channel, channel == g_currentChannel)) {
						g_currentChannel = channel;
					}
				}
				UI::EndCombo();
			}
			UI::SameLine();
			if (UI::Button("Clear")) {
				g_messages.RemoveRange(0, g_messages.Length);
				@g_currentMessage = null;
			}
			UI::Separator();
			if (UI::BeginTable("Messages", 5, UI::TableFlags::Resizable | UI::TableFlags::SizingStretchProp | UI::TableFlags::ScrollY)) {
				UI::TableSetupScrollFreeze(0, 1);
				UI::TableSetupColumn("Type");
				UI::TableSetupColumn("Channel");
				UI::TableSetupColumn("User");
				UI::TableSetupColumn("Message");
				UI::TableSetupColumn("Action");
				UI::TableHeadersRow();

				UI::ListClipper clipper(g_messages.Length);
				while (clipper.Step()) {
					for (int i = clipper.DisplayStart; i < clipper.DisplayEnd; i++) {
						UI::TableNextRow();

						auto message = g_messages[i];
						UI::PushID(message);

						UI::TableSetColumnIndex(0);
						UI::Text(MessageTypeString(message.m_messageType));

						UI::TableSetColumnIndex(1);
						UI::Text(message.m_channel);

						UI::TableSetColumnIndex(2);
						UI::Text(message.m_username);

						UI::TableSetColumnIndex(3);
						UI::Text(message.m_text);

						UI::TableSetColumnIndex(4);
						if (UI::Button(Icons::Tags + " Tags")) {
							@g_currentMessage = g_messages[i];
							g_showPopup = true;
						}

						UI::PopID();
					}
				}
				UI::EndTable();
			}
			UI::End();
		}
	}

	if (g_showPopup) {
		if (UI::Begin(Icons::Tags + " Message Info###Message Info", g_showPopup)) {
			if (g_currentMessage !is null) {
				if (UI::BeginTable("Attributes", 2, UI::TableFlags::Resizable)) {
					UI::TableSetupScrollFreeze(0, 1);
					UI::TableSetupColumn("Attribute");
					UI::TableSetupColumn("Value");
					UI::TableHeadersRow();
					UI::TableNextRow();
					UI::TableNextColumn(); UI::Text("m_type");
					UI::TableNextColumn(); UI::Text("" + int(g_currentMessage.m_messageType)); 
					UI::TableNextColumn(); UI::Text("m_channel"); 
					UI::TableNextColumn(); UI::Text(g_currentMessage.m_channel); 
					UI::TableNextColumn(); UI::Text("m_username"); 
					UI::TableNextColumn(); UI::Text(g_currentMessage.m_username); 
					UI::TableNextColumn(); UI::Text("m_text"); 
					UI::TableNextColumn(); UI::Text(g_currentMessage.m_text);
					UI::EndTable();
				}
				UI::Separator();
				if (UI::BeginTable("Tags", 2, UI::TableFlags::Resizable)) {
					UI::TableSetupScrollFreeze(0, 1);
					UI::TableSetupColumn("Tag");
					UI::TableSetupColumn("Value");
					UI::TableHeadersRow();
					for (uint i = 0; i < g_currentMessage.m_tags.GetSize(); i++) {
						UI::TableNextRow();

						UI::TableSetColumnIndex(0);
						UI::Text(g_currentMessage.m_tags.GetKeys()[i]);
						
						UI::TableSetColumnIndex(1);
						UI::Text(string(g_currentMessage.m_tags[g_currentMessage.m_tags.GetKeys()[i]]));
					}
					UI::EndTable();
				}
			} else {
				UI::Text("\\$f55No message selected!");
			}
			UI::End();
		}
	}
}
